#!/bin/sh

export LC_ALL=C

a= . install2-init-functions
SYSTEMCTL="/bin/systemctl --no-reload"
CHKCONFIG="/sbin/chkconfig"

# GUEST_TYPE: `man systemd-detect-virt`
GUEST_TYPE="none"
# services lists
KVM_GUEST="qemu-guest-agent.service"
KVM_HOST=
ORACLE_GUEST="vboxadd.service vboxadd-service.service vboxadd vboxadd-service"
ORACLE_HOST="vboxweb-service.service virtualbox.service vboxdrv.service vboxautostart-service.service vboxweb-service"
VMWARE_GUEST="vmtoolsd.service vgauthd.service"
VMWARE_HOST=

setup_service() {
    SYSTEM_SET=$1
    shift
    while [ -n "$1" ] ; do
	if [ "${1}" != "`echo "${1}"| tr -d '[.]'`" ] ; then
	    [ "$SYSTEM_SET" == "enable" ] && exec_chroot ${SYSTEMCTL} unmask ${1} 2>/dev/null ||:
	    exec_chroot ${SYSTEMCTL} ${SYSTEM_SET} ${1} 2>/dev/null ||:
	    [ "$SYSTEM_SET" == "disable" ] && exec_chroot ${SYSTEMCTL} mask ${1} 2>/dev/null ||:
	else
	    SYSV_SET=off
	    [ "$SYSTEM_SET" == "disable" ] || SYSV_SET=on
	    [ "$SYSV_SET" == "on" ] && exec_chroot ${CHKCONFIG} --add ${1} 2>/dev/null ||:
	    exec_chroot ${CHKCONFIG} ${1} ${SYSV_SET} 2>/dev/null ||:
	    [ "$SYSV_SET" == "off" ] && exec_chroot ${CHKCONFIG} --del ${1} 2>/dev/null ||:
	fi
	shift
    done
}

# detect virtual machine
SYSTEMD_DETECT_VIRT=`exec_chroot which systemd-detect-virt 2>/dev/null`
if [ -n "$SYSTEMD_DETECT_VIRT" ] ; then
    GUEST_TYPE=`exec_chroot $SYSTEMD_DETECT_VIRT --vm`
else
    if lspci -n | grep -qe '[[:space:]]1af4:100[[:alnum:]]' ; then
	GUEST_TYPE="kvm"
    fi
    if lspci -n | grep -qe '[[:space:]]15ad:07[[:alnum:]][[:alnum:]]' ; then
	GUEST_TYPE="vmware"
    fi
    if lspci -n | grep -qe '[[:space:]]80ee:beef' ; then
	GUEST_TYPE="oracle"
    fi
    if lspci -n | grep -qe '[[:space:]]1414:5353' ; then
	GUEST_TYPE="hyperv"
    fi
fi

if [ "$GUEST_TYPE" != "none" ] ; then
    # disable host services
    [ -z "$KVM_HOST" ]    || setup_service disable $KVM_HOST
    [ -z "$ORACLE_HOST" ]   || setup_service disable $ORACLE_HOST
    [ -z "$VMWARE_HOST" ] || setup_service disable $VMWARE_HOST
    # enable specific guest services
    case "$GUEST_TYPE" in
	kvm)
	    [ -z "$KVM_GUEST" ]    || setup_service enable $KVM_GUEST
	    [ -z "$VMWARE_GUEST" ] || setup_service disable $VMWARE_GUEST
	    [ -z "$ORACLE_GUEST" ]   || setup_service disable $ORACLE_GUEST
	    ;;
	vmware)
	    [ -z "$VMWARE_GUEST" ] || setup_service enable $VMWARE_GUEST
	    [ -z "$KVM_GUEST" ]    || setup_service disable $KVM_GUEST
	    [ -z "$ORACLE_GUEST" ]   || setup_service disable $ORACLE_GUEST
	    ;;
	oracle)
	    [ -z "$ORACLE_GUEST" ]   || setup_service enable $ORACLE_GUEST
	    [ -z "$KVM_GUEST" ]    || setup_service disable $KVM_GUEST
	    [ -z "$VMWARE_GUEST" ] || setup_service disable $VMWARE_GUEST
	    ;;
	*)
	    [ -z "$ORACLE_GUEST" ]   || setup_service disable $ORACLE_GUEST
	    [ -z "$KVM_GUEST" ]    || setup_service disable $KVM_GUEST
	    [ -z "$VMWARE_GUEST" ] || setup_service disable $VMWARE_GUEST
	    ;;
    esac
else
    # disable guest services
    [ -z "$KVM_GUEST" ]    || setup_service disable $KVM_GUEST
    [ -z "$ORACLE_GUEST" ]   || setup_service disable $ORACLE_GUEST
    [ -z "$VMWARE_GUEST" ] || setup_service disable $VMWARE_GUEST
    # enable host services
    [ -z "$KVM_HOST" ]    || setup_service enable $KVM_HOST
    [ -z "$ORACLE_HOST" ]   || setup_service enable $ORACLE_HOST
    [ -z "$VMWARE_HOST" ] || setup_service enable $VMWARE_HOST
fi
